==> fox0.99_0.99.175.orig.tar.gz: fox-0.99.175/src/FXMenubar.cpp <==
/********************************************************************************
*                                                                               *
*                           M e n u b a r   W i d g e t                         *
*                                                                               *
*********************************************************************************
* Copyright (C) 1998,2001 by Jeroen van der Zijp.   All Rights Reserved.        *
*********************************************************************************
* This library is free software; you can redistribute it and/or                 *
* modify it under the terms of the GNU Lesser General Public                    *
* License as published by the Free Software Foundation; either                  *
* version 2.1 of the License, or (at your option) any later version.            *
*                                                                               *
* This library is distributed in the hope that it will be useful,               *
* but WITHOUT ANY WARRANTY; without even the implied warranty of                *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             *
* Lesser General Public License for more details.                               *
*                                                                               *
* You should have received a copy of the GNU Lesser General Public              *
* License along with this library; if not, write to the Free Software           *
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
*********************************************************************************
* $Id: FXMenubar.cpp,v 1.14 2001/07/26 17:32:39 jeroen Exp $                    *
********************************************************************************/
#include "xincs.h"
#include "fxver.h"
#include "fxdefs.h"
#include "FXStream.h"
#include "FXString.h"
#include "FXSize.h"
#include "FXPoint.h"
#include "FXRectangle.h"
#include "FXObject.h"
#include "FXDict.h"
#include "FXSettings.h"
#include "FXRegistry.h"
#include "FXAccelTable.h"
#include "FXApp.h"
#include "FXId.h"
#include "FXDrawable.h"
#include "FXWindow.h"
#include "FXFrame.h"
#include "FXLabel.h"
#include "FXButton.h"
#include "FXComposite.h"
#include "FXPacker.h"
#include "FXToolbar.h"
#include "FXMenubar.h"


/*
  Notes:
  - Hittin Alt- key only should attract focus to the first item in the menubar.
  - If width of menu gets too small, expand the height to make it multiple rows.
  - FXMenubar should be renamed to FXMenubar.
  - FXMenubar should derive from FXToolbar, or at least allow for drag grip....
*/

/*******************************************************************************/


// Map
FXDEFMAP(FXMenubar) FXMenubarMap[]={
  FXMAPFUNC(SEL_ENTER,0,FXMenubar::onEnter),
  FXMAPFUNC(SEL_LEAVE,0,FXMenubar::onLeave),
  FXMAPFUNC(SEL_MOTION,0,FXMenubar::onMotion),
  FXMAPFUNC(SEL_LEFTBUTTONPRESS,0,FXMenubar::onButtonPress),
  FXMAPFUNC(SEL_LEFTBUTTONRELEASE,0,FXMenubar::onButtonRelease),
  FXMAPFUNC(SEL_MIDDLEBUTTONPRESS,0,FXMenubar::onButtonPress),
  FXMAPFUNC(SEL_MIDDLEBUTTONRELEASE,0,FXMenubar::onButtonRelease),
  FXMAPFUNC(SEL_RIGHTBUTTONPRESS,0,FXMenubar::onButtonPress),
  FXMAPFUNC(SEL_RIGHTBUTTONRELEASE,0,FXMenubar::onButtonRelease),
  FXMAPFUNC(SEL_FOCUS_RIGHT,0,FXMenubar::onFocusRight),
  FXMAPFUNC(SEL_FOCUS_LEFT,0,FXMenubar::onFocusLeft),
  FXMAPFUNC(SEL_FOCUS_NEXT,0,FXMenubar::onDefault),
  FXMAPFUNC(SEL_FOCUS_PREV,0,FXMenubar::onDefault),
  FXMAPFUNC(SEL_FOCUS_UP,0,FXMenubar::onDefault),
  FXMAPFUNC(SEL_FOCUS_DOWN,0,FXMenubar::onDefault),
  FXMAPFUNC(SEL_COMMAND,FXWindow::ID_UNPOST,FXMenubar::onCmdUnpost),
  };


// Object implementation
FXIMPLEMENT(FXMenubar,FXToolbar,FXMenubarMap,ARRAYNUMBER(FXMenubarMap))


// Make a floatable menubar
FXMenubar::FXMenubar(FXComposite* p,FXComposite* q,FXuint opts,FXint x,FXint y,FXint w,FXint h,FXint pl,FXint pr,FXint pt,FXint pb,FXint hs,FXint vs):
  FXToolbar(p,q,opts,x,y,w,h,pl,pr,pt,pb,hs,vs){
  flags|=FLAG_ENABLED;
  dragCursor=getApp()->getDefaultCursor(DEF_RARROW_CURSOR);
  }


// Make a non-floatable menubar
FXMenubar::FXMenubar(FXComposite* p,FXuint opts,FXint x,FXint y,FXint w,FXint h,FXint pl,FXint pr,FXint pt,FXint pb,FXint hs,FXint vs):
  FXToolbar(p,opts,x,y,w,h,pl,pr,pt,pb,hs,vs){
  flags|=FLAG_ENABLED;
  dragCursor=getApp()->getDefaultCursor(DEF_RARROW_CURSOR);
  }


// Focus moved to right
long FXMenubar::onFocusRight(FXObject*,FXSelector,void* ptr){
  FXWindow *child;
  if(getFocus()){
    child=getFocus()->getNext();
    while(child){
      if(child->isEnabled() && child->canFocus()){
        child->handle(this,MKUINT(0,SEL_FOCUS_SELF),ptr);
        return 1;
        }
      child=child->getNext();
      }
    child=getFirst();
    while(child){
      if(child->isEnabled() && child->canFocus()){
        child->handle(this,MKUINT(0,SEL_FOCUS_SELF),ptr);
        return 1;
        }
      child=child->getNext();
      }
    }
  return 0;
  }


// Focus moved to left
long FXMenubar::onFocusLeft(FXObject*,FXSelector,void* ptr){
  FXWindow *child;
  if(getFocus()){
    child=getFocus()->getPrev();
    while(child){
      if(child->isEnabled() && child->canFocus()){
        child->handle(this,MKUINT(0,SEL_FOCUS_SELF),ptr);
        return 1;
        }
      child=child->getPrev();
      }
    child=getLast();
    while(child){
      if(child->isEnabled() && child->canFocus()){
        child->handle(this,MKUINT(0,SEL_FOCUS_SELF),ptr);
        return 1;
        }
      child=child->getPrev();
      }
    }
  return 0;
  }


// Enter:- when inside the popup, all is normal!
long FXMenubar::onEnter(FXObject* sender,FXSelector sel,void* ptr){
  FXToolbar::onEnter(sender,sel,ptr);
  if(!getFocus() || !getFocus()->isActive()) return 1;
  if(((FXEvent*)ptr)->code==CROSSINGNORMAL){
    if(grabbed()) ungrab();
    }
  return 1;
  }


// Leave:- when outside the popup, a click will hide the popup!
long FXMenubar::onLeave(FXObject* sender,FXSelector sel,void* ptr){
  FXToolbar::onLeave(sender,sel,ptr);
  if(!getFocus() || !getFocus()->isActive()) return 1;
  if(((FXEvent*)ptr)->code==CROSSINGNORMAL){
    if(!grabbed()) grab();
    }
  return 1;
  }


// We're considered inside the menu bar when either
// in the bar or in any active menus
FXbool FXMenubar::contains(FXint parentx,FXint parenty) const {
  FXint x,y;
  if(FXComposite::contains(parentx,parenty)) return 1;
  if(getFocus()){
    getParent()->translateCoordinatesTo(x,y,this,parentx,parenty);
    if(getFocus()->contains(x,y)) return 1;
    }
  return 0;
  }


// Moved while outside
// We need to do this because the definition of ``inside'' means
// that we're inside even though possibly we're not in THIS window!!!
long FXMenubar::onMotion(FXObject*,FXSelector,void* ptr){
  FXEvent* ev=(FXEvent*)ptr;
  FXint px,py;
  if(!getFocus() || !getFocus()->isActive()) return 0;
  translateCoordinatesTo(px,py,getParent(),ev->win_x,ev->win_y);
  if(contains(px,py)){
    if(grabbed()) ungrab();
    }
  else{
    if(!grabbed()) grab();
    }
  return 0;
  }


// Button pressed
long FXMenubar::onButtonPress(FXObject*,FXSelector,void*){
  FXTRACE((200,"%s::onButtonPress %p\n",getClassName(),this));
  handle(this,MKUINT(ID_UNPOST,SEL_COMMAND),NULL);
  return 1;
  }


// Button released
long FXMenubar::onButtonRelease(FXObject*,FXSelector,void* ptr){
  FXEvent* ev=(FXEvent*)ptr;
  FXTRACE((200,"%s::onButtonRelease %p\n",getClassName(),this));
  if(ev->moved){ handle(this,MKUINT(ID_UNPOST,SEL_COMMAND),NULL); }
  return 1;
  }


// Unpost the menu
long FXMenubar::onCmdUnpost(FXObject*,FXSelector,void*){
  FXTRACE((200,"%s::onCmdUnpost %p\n",getClassName(),this));
  if(getFocus()) getFocus()->killFocus();
  //killFocus();
  return 1;
  }

==> fox_1.0.52.orig.tar.gz: fox-1.0.52/src/FXMenubar.cpp <==
/********************************************************************************
*                                                                               *
*                           M e n u b a r   W i d g e t                         *
*                                                                               *
*********************************************************************************
* Copyright (C) 1998,2002 by Jeroen van der Zijp.   All Rights Reserved.        *
*********************************************************************************
* This library is free software; you can redistribute it and/or                 *
* modify it under the terms of the GNU Lesser General Public                    *
* License as published by the Free Software Foundation; either                  *
* version 2.1 of the License, or (at your option) any later version.            *
*                                                                               *
* This library is distributed in the hope that it will be useful,               *
* but WITHOUT ANY WARRANTY; without even the implied warranty of                *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             *
* Lesser General Public License for more details.                               *
*                                                                               *
* You should have received a copy of the GNU Lesser General Public              *
* License along with this library; if not, write to the Free Software           *
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
*********************************************************************************
* $Id: FXMenubar.cpp,v 1.20 2002/01/18 22:43:01 jeroen Exp $                    *
********************************************************************************/
#include "xincs.h"
#include "fxver.h"
#include "fxdefs.h"
#include "FXStream.h"
#include "FXString.h"
#include "FXSize.h"
#include "FXPoint.h"
#include "FXRectangle.h"
#include "FXRegistry.h"
#include "FXAccelTable.h"
#include "FXApp.h"
#include "FXButton.h"
#include "FXMenubar.h"


/*
  Notes:
  - Hittin Alt- key only should attract focus to the first item in the menubar.
  - If width of menu gets too small, expand the height to make it multiple rows.
  - FXMenubar should be renamed to FXMenubar.
  - FXMenubar should derive from FXToolbar, or at least allow for drag grip....
*/

/*******************************************************************************/


// Map
FXDEFMAP(FXMenubar) FXMenubarMap[]={
  FXMAPFUNC(SEL_ENTER,0,FXMenubar::onEnter),
  FXMAPFUNC(SEL_LEAVE,0,FXMenubar::onLeave),
  FXMAPFUNC(SEL_MOTION,0,FXMenubar::onMotion),
  FXMAPFUNC(SEL_LEFTBUTTONPRESS,0,FXMenubar::onButtonPress),
  FXMAPFUNC(SEL_LEFTBUTTONRELEASE,0,FXMenubar::onButtonRelease),
  FXMAPFUNC(SEL_MIDDLEBUTTONPRESS,0,FXMenubar::onButtonPress),
  FXMAPFUNC(SEL_MIDDLEBUTTONRELEASE,0,FXMenubar::onButtonRelease),
  FXMAPFUNC(SEL_RIGHTBUTTONPRESS,0,FXMenubar::onButtonPress),
  FXMAPFUNC(SEL_RIGHTBUTTONRELEASE,0,FXMenubar::onButtonRelease),
  FXMAPFUNC(SEL_FOCUS_RIGHT,0,FXMenubar::onFocusRight),
  FXMAPFUNC(SEL_FOCUS_LEFT,0,FXMenubar::onFocusLeft),
  FXMAPFUNC(SEL_FOCUS_NEXT,0,FXMenubar::onDefault),
  FXMAPFUNC(SEL_FOCUS_PREV,0,FXMenubar::onDefault),
  FXMAPFUNC(SEL_FOCUS_UP,0,FXMenubar::onDefault),
  FXMAPFUNC(SEL_FOCUS_DOWN,0,FXMenubar::onDefault),
  FXMAPFUNC(SEL_COMMAND,FXWindow::ID_UNPOST,FXMenubar::onCmdUnpost),
  };


// Object implementation
FXIMPLEMENT(FXMenubar,FXToolbar,FXMenubarMap,ARRAYNUMBER(FXMenubarMap))


// Make a floatable menubar
FXMenubar::FXMenubar(FXComposite* p,FXComposite* q,FXuint opts,FXint x,FXint y,FXint w,FXint h,FXint pl,FXint pr,FXint pt,FXint pb,FXint hs,FXint vs):
  FXToolbar(p,q,opts,x,y,w,h,pl,pr,pt,pb,hs,vs){
  flags|=FLAG_ENABLED;
  dragCursor=getApp()->getDefaultCursor(DEF_RARROW_CURSOR);
  }


// Make a non-floatable menubar
FXMenubar::FXMenubar(FXComposite* p,FXuint opts,FXint x,FXint y,FXint w,FXint h,FXint pl,FXint pr,FXint pt,FXint pb,FXint hs,FXint vs):
  FXToolbar(p,opts,x,y,w,h,pl,pr,pt,pb,hs,vs){
  flags|=FLAG_ENABLED;
  dragCursor=getApp()->getDefaultCursor(DEF_RARROW_CURSOR);
  }


// Focus moved to right
long FXMenubar::onFocusRight(FXObject*,FXSelector,void* ptr){
  FXWindow *child;
  if(getFocus()){
    child=getFocus()->getNext();
    while(child){
      if(child->isEnabled() && child->canFocus()){
        child->handle(this,MKUINT(0,SEL_FOCUS_SELF),ptr);
        return 1;
        }
      child=child->getNext();
      }
    child=getFirst();
    while(child){
      if(child->isEnabled() && child->canFocus()){
        child->handle(this,MKUINT(0,SEL_FOCUS_SELF),ptr);
        return 1;
        }
      child=child->getNext();
      }
    }
  return 0;
  }


// Focus moved to left
long FXMenubar::onFocusLeft(FXObject*,FXSelector,void* ptr){
  FXWindow *child;
  if(getFocus()){
    child=getFocus()->getPrev();
    while(child){
      if(child->isEnabled() && child->canFocus()){
        child->handle(this,MKUINT(0,SEL_FOCUS_SELF),ptr);
        return 1;
        }
      child=child->getPrev();
      }
    child=getLast();
    while(child){
      if(child->isEnabled() && child->canFocus()){
        child->handle(this,MKUINT(0,SEL_FOCUS_SELF),ptr);
        return 1;
        }
      child=child->getPrev();
      }
    }
  return 0;
  }


// Enter:- when inside the popup, all is normal!
long FXMenubar::onEnter(FXObject* sender,FXSelector sel,void* ptr){
  FXEvent* ev=(FXEvent*)ptr;
  FXint px, py;
  FXToolbar::onEnter(sender,sel,ptr);
  if(!getFocus() || !getFocus()->isActive()) return 1;
  if(((FXEvent*)ptr)->code==CROSSINGNORMAL){
    translateCoordinatesTo(px,py,getParent(),ev->win_x,ev->win_y);
    if(contains(px,py) && grabbed()) ungrab();
    //if(grabbed()) ungrab();
    }
  return 1;
  }


// Leave:- when outside the popup, a click will hide the popup!
long FXMenubar::onLeave(FXObject* sender,FXSelector sel,void* ptr){
  FXEvent* ev=(FXEvent*)ptr;
  FXint px,py;
  FXToolbar::onLeave(sender,sel,ptr);
  if(!getFocus() || !getFocus()->isActive()) return 1;
  if(((FXEvent*)ptr)->code==CROSSINGNORMAL){
    translateCoordinatesTo(px,py,getParent(),ev->win_x,ev->win_y);
    if(!contains(px,py) && !grabbed()) grab();
//#ifndef WIN32
//    if(!grabbed()) grab();
//#endif
    }
  return 1;
  }


// We're considered inside the menu bar when either
// in the bar or in any active menus
FXbool FXMenubar::contains(FXint parentx,FXint parenty) const {
  FXint x,y;
  if(FXComposite::contains(parentx,parenty)) return 1;
  if(getFocus()){
    getParent()->translateCoordinatesTo(x,y,this,parentx,parenty);
    if(getFocus()->contains(x,y)) return 1;
    }
  return 0;
  }


// Moved while outside
// We need to do this because the definition of ``inside'' means
// that we're inside even though possibly we're not in THIS window!!!
long FXMenubar::onMotion(FXObject*,FXSelector,void* ptr){
  FXEvent* ev=(FXEvent*)ptr;
  FXint px,py;
  if(!getFocus() || !getFocus()->isActive()) return 0;
  translateCoordinatesTo(px,py,getParent(),ev->win_x,ev->win_y);
  if(contains(px,py)){
    if(grabbed()) ungrab();
    }
  else{
    if(!grabbed()) grab();
    }
  return 0;
  }


// Button pressed
long FXMenubar::onButtonPress(FXObject*,FXSelector,void*){
  FXTRACE((200,"%s::onButtonPress %p\n",getClassName(),this));
  handle(this,MKUINT(ID_UNPOST,SEL_COMMAND),NULL);
  return 1;
  }


// Button released
long FXMenubar::onButtonRelease(FXObject*,FXSelector,void* ptr){
  FXEvent* ev=(FXEvent*)ptr;
  FXTRACE((200,"%s::onButtonRelease %p\n",getClassName(),this));
  if(ev->moved){ handle(this,MKUINT(ID_UNPOST,SEL_COMMAND),NULL); }
  return 1;
  }


// Unpost the menu
long FXMenubar::onCmdUnpost(FXObject*,FXSelector,void*){
  FXTRACE((200,"%s::onCmdUnpost %p\n",getClassName(),this));
  if(getFocus()) getFocus()->killFocus();
  //killFocus();
  return 1;
  }
